 /*
    Peter's Picross - A remake for TI 68k-based calculators of the Nintendo Gameboy game, "Mario's Picross"
    Copyright (C) 2008  Peter Fernandes
    supersonicandtails@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    
    
    editor.c: dialog for creating a new puzzle
*/

#include <tigcclib.h>
#include "main.h"
#include "engine.h"
#include "editor.h"
#include "files.h"
#include "gfx.h"


short create_puzzle(short generate_random) {
	char name[21];
	char author[21];
	char filename[9];
	char dim[2];
	char dimint;
	PUZZLE puzzle;
	char newstr[18];
	short board[15];
	short i=0;
	
	
	memset(board,0,sizeof(board));
	memset(newstr,0,sizeof(newstr));
	strcpy(author,settings_file);
	
	
	memcpy(newstr+1,"picross\\",8);
	
	FontSetSys(F_6x8);
	
	
	ClrScr();
	DrawStr((LCD_WIDTH-(12*7))/2,0,"Create Puzzle",A_NORMAL);
	DrawStr(10,15,"Your Name:",A_NORMAL);
	DrawStr(10,35,"Puzzle Title:",A_NORMAL);
	DrawStr(10,55,"Dimensions (5-15):",A_NORMAL);
	DrawStr(10,75,"Filename:",A_NORMAL);
	
	
	//draw the stored name in
	if(settings_file) {
		i=strlen(settings_file);
		
		DrawStr(10,25,settings_file,A_NORMAL);
	}
	
	
	
	//your name                //puzzle name         //puzzle dimensions //requested file name
	if(getstr(author,20,25,1,i) || getstr(name,20,45,1,0))
		return -1;
		
	i=0;
	
	enter_dim:
	
	*((unsigned short *)dim)=0;
	
	if(getstr(dim,2,65,0,0))
		return -1;
		
	
	
	//convert the char to an int value
	if(dim[1])
		dimint=((dim[0]-'0')*10)+dim[1]-'0';
	else
		dimint=dim[0]-'0';
	
	if(dimint<5 || dimint >15) {
		DrawStr(10,65,"Must be 5<=dim<=15",A_REPLACE);
		ngetchx();
		
		ScrRectFill(&(SCR_RECT){{10,65,18*7,73}},&(SCR_RECT){{10,65,18*7,73}},A_REVERSE);
		goto enter_dim;
	}
	
	
	enter_filename:
	
	if(getstr(filename,8,85,0,0))
		return -1;
	
	if(ValidateSymName(filename)==FALSE) {
		DrawStr(10,75,"Invalid Filename",A_REPLACE);
		ngetchx();
		
		ScrRectFill(&(SCR_RECT){{10,85,11*7,93}},&(SCR_RECT){{10,85,11*7,93}},A_REVERSE);
		goto enter_filename;
	}
		
	//make sure the filename isn't already taken	
	memcpy(newstr+9,filename,strlen(filename));
	
	
	
	if(SymFind(newstr+strlen(newstr+1)+1).folder) {
		DrawStr(10,85,"File exists",A_REPLACE);
		ngetchx();
		
		ScrRectFill(&(SCR_RECT){{10,85,11*7,93}},&(SCR_RECT){{10,85,11*7,93}},A_REVERSE);
		goto enter_filename;
	}
	
		
	
	
	puzzle.name=name;
	puzzle.author=author;
	puzzle.dim=dimint;
	
	
	//store creator name for recall later
	if(settings_file) {
		strcpy(settings_file,author);
	}
	
	
	if(!generate_random) {
		ClrScr();
		if((i=main_loop(&puzzle,MODE_EDIT,board))==-1)
			return -6;
		else if(i)
			return -1;
	}
	else {
		//create random board
		for(dimint=0;dimint<puzzle.dim;dimint++) {
			for(i=0;i<puzzle.dim;i++) {
				board[(short)dimint]|=random(2)<<i;
			}
		}
		
	}
	
	
	puzzle.board=board;
	
	switch(write_puzzle_file(filename,&puzzle)) {
		case -3:
			return -3;
		case -2:
			return -1;
		case -4:
			return -4;
		case -5:
			return -5;
	}
	
	
	return 0;
}



//gets a string from the user.  Returns -1 is ESC was pressed, 0 otherwise
short getstr(char *buf, short limit, short y, char allowspace, short offset) {
	unsigned int key;
	short pos=offset;
	
	Sprite8(0,y+1,6,input_arrow,LCD_MEM,SPRT_OR);
	
	top:
	
	while((key=ngetchx())!=KEY_ENTER) {
		switch (key) {
			case ' ' ... '~':
				if(pos<limit && !((!allowspace) && key==' ')) {
					buf[pos]=key;
					DrawChar(pos*6+10,y,key,A_NORMAL);
					pos++;
				}
				break;
			case KEY_ESC:
				return -1;
			case 257: //bksp
				if(pos) {
					pos--;
					DrawChar(pos*6+10,y,' ',A_REPLACE);
				}
				break;
		}
		
	}
	
	if(!pos)
		goto top;
	
	//null char
	if(limit>2)
		buf[pos]=0;
		
	Sprite8(0,y+1,6,input_arrow_mask,LCD_MEM,SPRT_AND);
	
	return 0;
}

