/*
Peter's Picross Puzzle Editor - For computer-based creation of Peter's Picross puzzle files
Copyright (C) 2008  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tilp2/tifiles.h>


//assuming this is a little-endian machine...
#define swap(x) (x<8?x+8:x-8)


//moves down a line - returns -1 on a file error or EOF
int down_line(FILE *file) {
	while(fgetc(file)!='\n') {
		if(ferror(file) || feof(file))
			return -1;
	}
	
	return 0;
}


int main(int argc, char **argv) {
	FILE *in;
	unsigned int h;
	unsigned int w;
	char *ptr;
	int i;

	char name[21];
	char title[21];
	char filename[9];
	int dim;
	short board[15];
	char temp;
	
	FileContent *tifile;
	VarEntry *entry;
	
	
	tifiles_library_init();
	
	
	if(argc<=1) {
		fprintf(stderr,"Error: no input files\n");
		goto end3;
	}
	
	
	for(i=0;i<argc-1;i++) {
	
		if(!(in=fopen(argv[i+1],"rb"))) {
			fprintf(stderr,"%s: Error opening input file\n",argv[i+1]);
			continue;
		}
		
		
		memset(board,0,sizeof(board));
		

		//check for file identifier "ppp"
		fgets(name,4,in);


		if(strcmp(name,"ppp")) {
			fprintf(stderr,"Ignoring %s: indentifier string not found\n",argv[i+1]);
			goto error_justclose;
		}

		fseek(in,1,SEEK_CUR);
		
		fgets(name,21,in);


		if((*(name+strlen(name)-1)!='\n') && down_line(in)) {
			goto error_eof;
		}
		
		*(name+strlen(name)-1)=0;
		
		
		if(*name==0) {
			fprintf(stderr,"%s: Parse error [line 2]: author line not found\n",argv[i+1]);
			goto error_justclose;
		}
		
		
		
		
		
		fgets(title,21,in);
		
		if((*(title+strlen(title)-1)!='\n') && down_line(in)) {
			goto error_eof;
		}
		
		*(title+strlen(title)-1)=0;
		
		
		if(*title==0) {
			fprintf(stderr,"%s: Parse error [line 3]: title line not found\n",argv[i+1]);
			goto error_justclose;
		}
	
		
		fscanf(in,"%d",&dim);
		
		if(down_line(in)) {
			goto error_eof;
		}
		
		
		fgets(filename,9,in);
		
		
		if((*(filename+strlen(filename)-1)!='\n') && down_line(in)) {
			goto error_justclose;
		}
		
		*(filename+strlen(filename)-1)=0;
		
		
		
		
		if(*filename==0) {
			fprintf(stderr,"%s: Parse error [line 5]: filename line not found\n",argv[i+1]);
			goto error_justclose;
		}
		
		
		
		
		
		tifile=tifiles_content_create_regular(CALC_TI89);
		
		
		tifile->model=CALC_TI89;
		*tifile->default_folder=0;
		strcpy(tifile->comment,"Peter's Picross Puzzle");
		tifile->num_entries=1;
		
		
		tifile->entries=tifiles_ve_create_array(2);
		tifile->entries[0]=entry=tifiles_ve_create();
		
		//now fill the VarEntry with the info
		*entry->folder=0;
		entry->type=0x1C; //type id
		entry->attr=ATTRB_NONE;
		entry->action=0;
		entry->size=9+48+(dim*2);
		strcpy(entry->name,filename);
		
		entry->data=tifiles_ve_alloc_data(9+48+(dim*2));
		
		
		
		//get board data
		for(h=0;h<dim;h++) {
			for(w=0;w<dim;w++) {
				board[h]|=((temp=fgetc(in))=='1'?1:0)<<swap(dim-w-1);
				
				if(temp=='\n') {
					fprintf(stderr,"%s: Parse error [line %d]: newline encountered early\n",argv[i+1],h+6);
					goto end2;
				}
			}
			
			//move to the next line
			if(h<(dim-1) && down_line(in)) {
				fprintf(stderr,"%s: Parse error: end of file reached\n",argv[i+1]);
				goto end2;
			}
		}
		
		
		
		ptr=(char *)(entry->data+1);
		
		//start writing the puzzle data
		*((char *)ptr)=entry->size-2;
		
		ptr++;
		
		//puzzle file identifier 0x70706200	
		*(ptr++)=0x70;
		*(ptr++)=0x70;
		*(ptr++)=0x62;
		*(ptr++)=0x0;
		
		
		strcpy(ptr,title);
		ptr+=21;
		strcpy(ptr,name);
		ptr+=21;
		
		*ptr=(char)dim;
		
		ptr+=2;
		
		memcpy(ptr,board,dim*2);
		
		
		
		ptr+=(dim*2);
		
		*(ptr++)=0;
		*(ptr++)='P';
		*(ptr++)='I';
		*(ptr++)='C';
		*(ptr++)='B';
		*(ptr++)=0;
		*(ptr)=0xF8;
		
		for(w=0,ptr=(char *)entry->data;w<entry->size;w++,ptr++) {
			tifile->checksum+=*ptr;
		}
		
		
		if(tifiles_file_write_regular(NULL,tifile,&ptr)) {
			fprintf(stderr,"Error writing TI File");
		}
		
		end2:
		
		tifiles_content_delete_regular(tifile);
		fclose(in);
		
		continue;
		
		error_eof:
	
		fprintf(stderr,"%s: Parse error: end of file reached\n",argv[i+1]);

		error_justclose:
		fclose(in);
	
	}

	tifiles_library_exit();
	
	end3:
	
	return 0;
}


